﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading; 
using Arction.WPF.LightningChartUltimate;
using Arction.WPF.LightningChartUltimate.Views;
using Arction.WPF.LightningChartUltimate.Views.ViewXY;
using Arction.WPF.LightningChartUltimate.SeriesXY; 

namespace WpfApplication1
{
    /// <summary>
    /// Interaction logic for PersistentPlot.xaml
    /// </summary>
    public partial class PersistentPlot : UserControl
    {
        LightningChartUltimate chart;
        PersistentSeriesRenderingIntensityLayer historicDataLayer;
        const double XMax = 8000;
        const double XMin = 0;
        const double YMin = 0;
        const double YAmplitudeMin = 100;
        const double YMax = 200;
        const double YAmplitudeMax = 200;

        const int PointCountPerTrace = 100;
        DispatcherTimer timer;
        int timerCounter = 0;
        int traceCountToPlot;

        public PersistentPlot(int count, double historicIntensityFactor)
        {
            InitializeComponent();

            textBoxTraceCount.Text = count.ToString();
            textBoxHIF.Text = historicIntensityFactor.ToString("0.0");

            chart = new LightningChartUltimate();
            gridMain.Children.Add(chart);
            Grid.SetRow(chart, 1);


            chart.BeginUpdate();

            chart.ViewXY.XAxes[0].SetRange(XMin, XMax);
            chart.ViewXY.XAxes[0].ValueType = AxisValueType.Number; 
            chart.ViewXY.YAxes[0].SetRange(YMin, YMax);

            PointLineSeries plsStart = new PointLineSeries(chart.ViewXY, chart.ViewXY.XAxes[0], chart.ViewXY.YAxes[0]);
            plsStart.LineStyle.Pattern = LinePattern.Dash;
            plsStart.LineStyle.Color = Colors.Magenta; 
            plsStart.ShowInLegendBox = false; 
            chart.ViewXY.PointLineSeries.Add(plsStart);


            PointLineSeries plsNow = new PointLineSeries(chart.ViewXY, chart.ViewXY.XAxes[0], chart.ViewXY.YAxes[0]);
            plsNow.LineStyle.Pattern = LinePattern.Solid;
            plsNow.ShowInLegendBox = false;
            plsNow.LineStyle.Color = Colors.Magenta; 
            chart.ViewXY.PointLineSeries.Add(plsNow);

            chart.ViewXY.LegendBox.Layout = LegendBoxLayout.Vertical;
            chart.ViewXY.LegendBox.ShowCheckboxes = false; 

            CreatePersistentLayer(); 

            chart.EndUpdate();


            timer = new DispatcherTimer();
            timer.Interval = TimeSpan.FromMilliseconds(10); 
            timer.Tick += timer_Tick;
            

        }

        void timer_Tick(object sender, EventArgs e)
        {
            timerCounter++; 


            chart.BeginUpdate();

            chart.Title.Text = timerCounter.ToString();
            
            HighLowSeries series = new HighLowSeries(chart.ViewXY, chart.ViewXY.XAxes[0], chart.ViewXY.YAxes[0]);
            
            //Actually color doesn't matter here
            series.LineVisible = false;

            double xInterval = (XMax - XMin) / (double)(PointCountPerTrace-1);
            double yInterval = (YAmplitudeMax - YAmplitudeMin) / (double)(traceCountToPlot-1); 

            HighLowSeriesPoint[] points = new HighLowSeriesPoint[PointCountPerTrace];
            SeriesPoint[] linePoints = new SeriesPoint[PointCountPerTrace];
            

            double x, y; 
            for (int i = 0; i < PointCountPerTrace; i++)
            {
                x = XMin + (double)i * xInterval;
                y = YAmplitudeMax - yInterval*(double)timerCounter;
                
                points[i].X = x;
                y *= Math.Sin(x / 1000.0);
                points[i].YHigh = y; 
                points[i].YLow = 0;

                linePoints[i].X = x;
                linePoints[i].Y = y; 

               

            }
            series.Points = points;

            if (timerCounter == 1)
            {
                chart.ViewXY.PointLineSeries[0].Points = linePoints;
            }

            chart.ViewXY.PointLineSeries[1].Points = linePoints; 


            historicDataLayer.RenderSeries(series); 

            chart.EndUpdate();

            if (timerCounter >= traceCountToPlot)
                timer.Stop(); 
            
        }

        public void Plot(int count, double historicIntensityFactor)
        {
            textBoxTraceCount.Text = count.ToString();
            textBoxHIF.Text = historicIntensityFactor.ToString("0.0");
            timerCounter = 0;
            traceCountToPlot = count; 

            chart.BeginUpdate();

            historicDataLayer.Clear();
            historicDataLayer.HistoryIntensityFactor = historicIntensityFactor;
            
            chart.EndUpdate();

            timer.Start(); 
        }

        private void CreatePersistentLayer()
        {
            chart.BeginUpdate();
            historicDataLayer = new PersistentSeriesRenderingIntensityLayer(chart.ViewXY, chart.ViewXY.XAxes[0]);
            
            historicDataLayer.NewTraceIntensity = 1;
            historicDataLayer.HistoryIntensityFactor = 1;
            historicDataLayer.AntiAliasing = false; 
            historicDataLayer.ValueRangePalette.Type = PaletteType.Gradient;
            
            ValueRangePalette palette = new ValueRangePalette(historicDataLayer);
            palette.Steps.Clear();
            palette.MinValue = 0;
            
            int colorCount = 8;
            double startValue = 0;
            double endValue = 100;
            double stepSize = (endValue - startValue) / (double)colorCount;
            double currentStep = startValue - stepSize;
            List<PaletteStep> steps = new List<PaletteStep>();
            steps.Add(new PaletteStep(palette, Color.FromArgb(0,0,0,0), 0));
            steps.Add(new PaletteStep(palette, Colors.Blue, 0.00001));
            steps.Add(new PaletteStep(palette, Colors.Green, 25));
            steps.Add(new PaletteStep(palette, Colors.Yellow, 50));
            steps.Add(new PaletteStep(palette, Colors.Red, 100));
            palette.Steps = new Arction.WPF.LightningChartUltimate.Views.ViewXY.PaletteStepObservableCollection(steps);
            palette.Type = PaletteType.Gradient;

            historicDataLayer.ValueRangePalette = palette;

            //Set legend box related properties
            historicDataLayer.LegendBoxValueFormat = "0";
            historicDataLayer.LegendBoxValueType = AxisValueType.Number;
            historicDataLayer.Title.Text = "Hit count";
            historicDataLayer.UnitsText = "";

            chart.EndUpdate();
        }

        private void buttonRenderTraces_Click(object sender, RoutedEventArgs e)
        {
            Plot(int.Parse(textBoxTraceCount.Text), double.Parse(textBoxHIF.Text));
        }

    }
}
